/**************************************************************

   Program Title:  IMU6410 Controller
   
   Program Name:  IMU6410_DAD.c	  Version: 0.97

   Compiled with AVR Studio 4.19
   
   This verison is designed for the ATmega1284P processor:
   		128K Flash, 4K EEPROM, 16K RAM
		
   Fuse bit:  FF D9 EE  -External OSC 
	
   This program setup a command sequence that responds to configuration
   commands set to from the DAD Desktop Application.  

   
   Copyright 2004-2011, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2011, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Robotics, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU6410_DAD.c,v $
 * Revision 0.97  2012/01/02 16:40:20  SS
 *    Kernel for DAD interaction added.
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 * 
 */ 
// Include header files for GCC/IAR
#include "avr_compiler.h"
#include <avr/pgmspace.h>

#include <string.h>
//#include <stdlib.h>
#include <math.h>
#include <stdio.h>
//#include <AVRdef.h>

#include "IMU6410_20_Uart.h"
#include "IMU6410_20_Defines.h"
#include "IMU6410_20_Extern.h"

// ADC0, ADC1 and ADC 2
void Write_Test_Data()
{
 	 WriteEepromInt(10,1);
	 WriteEepromInt(12,1);
	 
 	 // ADC0
	 EEPROM_write(70,0x01);
	 WriteEepromInt(30,10);
	 WriteEepromInt(32,10);

	 // ADC1
	 EEPROM_write(71,0x01);
	 WriteEepromInt(34,10);
	 WriteEepromInt(36,10);
	
	 // ADC5  
	 EEPROM_write(75,0x01);
	 WriteEepromInt(50,10);
	 WriteEepromInt(52,10);
}


void Dump_Init_Channel_Config()
{
     // Based on the values stored in EEPROM sets all the variables
 	 startCondition = ReadEepromInt(10);
	 EndCondition = ReadEepromInt(12);
	 printf("\n\rStart %x End %x\n\r",startCondition,EndCondition);
	 	  
	 StartPort = ReadEepromInt(14);
	 StartPin = ReadEepromInt(16);
	 printf("Start port %x Start Pin %x\n\r",StartPort,StartPin);
	 
	 StartThresholdValue = ReadEepromInt(18);
	 StartThresholdPin = ReadEepromInt(20); 	  
	 printf("Start Threshold %x Start Pin %x\n\r",StartThresholdValue,StartThresholdPin);
	 
	 StopPort = ReadEepromInt(22);
	 StopPin = ReadEepromInt(24);
	 printf("Stop port %x Stop Pin %x\n\r",StopPort,StopPin);
	 
	 StopThresholdValue = ReadEepromInt(26);
	 StopThresholdPin = ReadEepromInt(28);
	 printf("Stop Threshold %x Stop Pin %x\n\r",StopThresholdValue,StopThresholdPin);
	 
	// "70" is where the flag for the sampling ADC0 is stored on EEPROM
	// if that flag(at 70) ==1 then set ADC0 Exists to 1. Load resolution and sampleRate from 
	//associated location. Look for "EEPROM map.doc" for details of these locations	 
	// if flag==0 then it means sampling of ADC0 should be disabled, Hence ADC0Exists=0;
	
	 printf("\n\rADC0 %x",EEPROM_read(70));
	
	 if(EEPROM_read(70)==0x01)	 {
	     ADC0Exists = 1;
	     ADC0Resolution = ReadEepromInt(30);
  	     SampleRate0 = ReadEepromInt(32);
		 printf(" %d %d %d %d",ADC0Resolution,SampleRate0,ADC0Resolution,SampleRate0);
	     }
	 else
	     {ADC0Exists=0;}
	  
	 printf("\n\rADC1 %x",EEPROM_read(71));
	 if(EEPROM_read(71)==0x01)	 {	
	     ADC1Exists = 1;
	     ADC1Resolution = ReadEepromInt(34);
  	     SampleRate1 = ReadEepromInt(36);					  
		 printf(" %d %d",ADC1Resolution,SampleRate1);
	     }
	 else
	     {ADC1Exists=0;}
	 
	 printf("\n\rADC2 %x",EEPROM_read(72));
	 if(EEPROM_read(72)==0x01)	 {	
	     ADC2Exists = 1;
	     ADC2Resolution = ReadEepromInt(38);
  	     SampleRate2 = ReadEepromInt(40);					  
		 printf(" %d %d",ADC2Resolution,SampleRate2);
	     }
	 else
	     {ADC2Exists=0;}
	 
	 printf("\n\rADC3 %x",EEPROM_read(73));
	 if(EEPROM_read(73)==0x01)	 {  
	     ADC3Exists = 1;
	     ADC3Resolution = ReadEepromInt(42);
  	     SampleRate3 = ReadEepromInt(44);
		 printf(" %d %d",ADC3Resolution,SampleRate3);
	     }
	 else
	     {ADC3Exists=0;}
	 
	 printf("\n\rADC4 %x",EEPROM_read(74));
	 if(EEPROM_read(74)==0x01)	 {  
	     ADC4Exists = 1;
	     ADC4Resolution = ReadEepromInt(46);
  	     SampleRate4 = ReadEepromInt(48);
		 printf(" %d %d",ADC4Resolution,SampleRate4);
	     }
	 else
	     {ADC4Exists=0;}
	 
	 printf("\n\rADC5 %x",EEPROM_read(75));
	 if(EEPROM_read(75)==0x01)	 {  
	     ADC5Exists = 1;
	     ADC5Resolution = ReadEepromInt(50);
  	     SampleRate5 = ReadEepromInt(52);  
		 printf(" %d %d",ADC5Resolution,SampleRate5);
	     }
	 else
	     {ADC5Exists=0;}
	 
	 printf("\n\rADC6 %x",EEPROM_read(76));
	 if(EEPROM_read(76)==0x01)	 {  
	     ADC6Exists = 1;
	     ADC6Resolution = ReadEepromInt(54);
  	     SampleRate6 = ReadEepromInt(56);
		 printf(" %d %d",ADC6Resolution,SampleRate6);
	     }
	 else
	     {ADC6Exists=0;}
	 
	 printf("\n\rADC7 %x",EEPROM_read(77));
	 if(EEPROM_read(77)==0x01)	 {   
	     ADC7Exists = 1;
	     ADC7Resolution = ReadEepromInt(58);
  	     SampleRate7 = ReadEepromInt(60);
		 printf(" %d %d",ADC7Resolution,SampleRate7);
	     }
	 else
	     {ADC7Exists = 0;}
	 printf("\n\r");
}

void Init_Channel_Config()
{
     // Based on the values stored in EEPROM sets all the variables
 	 startCondition = ReadEepromInt(10);
	 EndCondition = ReadEepromInt(12);
	 	  
	 StartPort = ReadEepromInt(14);
	 StartPin = ReadEepromInt(16);
	 
	 StartThresholdValue = ReadEepromInt(18);
	 StartThresholdPin = ReadEepromInt(20); 	  
	 
	 StopPort = ReadEepromInt(22);
	 StopPin = ReadEepromInt(24);
	 
	 StopThresholdValue = ReadEepromInt(26);
	 StopThresholdPin = ReadEepromInt(28);
	 
	// "70" is where the flag for the sampling ADC0 is stored on EEPROM
	// if that flag(at 70) ==1 then set ADC0 Exists to 1. Load resolution and sampleRate from 
	//associated location. Look for "EEPROM map.doc" for details of these locations	 
	// if flag==0 then it means sampling of ADC0 should be disabled, Hence ADC0Exists=0;
	 if(EEPROM_read(70)==0x01)	 {
	     ADC0Exists = 1;
	     ADC0Resolution = ReadEepromInt(30);
  	     SampleRate0 = ReadEepromInt(32);
	     }
	 else
	     {ADC0Exists=0;}
	  
	 if(EEPROM_read(71)==0x01)	 {	
	     ADC1Exists = 1;
	     ADC1Resolution = ReadEepromInt(34);
  	     SampleRate1 = ReadEepromInt(36);					  
	     }
	 else
	     {ADC1Exists=0;}
	 
	 if(EEPROM_read(72)==0x01)	 {	
	     ADC2Exists = 1;
	     ADC2Resolution = ReadEepromInt(38);
  	     SampleRate2 = ReadEepromInt(40);					  
	     }
	 else
	     {ADC2Exists=0;}
	 
	 if(EEPROM_read(73)==0x01)	 {  
	     ADC3Exists = 1;
	     ADC3Resolution = ReadEepromInt(42);
  	     SampleRate3 = ReadEepromInt(44);
	     }
	 else
	     {ADC3Exists=0;}
	 
	 if(EEPROM_read(74)==0x01)	 {  
	     ADC4Exists = 1;
	     ADC4Resolution = ReadEepromInt(46);
  	     SampleRate4 = ReadEepromInt(48);
	     }
	 else
	     {ADC4Exists=0;}
	 
	 if(EEPROM_read(75)==0x01)	 {  
	     ADC5Exists = 1;
	     ADC5Resolution = ReadEepromInt(50);
  	     SampleRate5 = ReadEepromInt(52);  
	     }
	 else
	     {ADC5Exists=0;}
	 
	 if(EEPROM_read(76)==0x01)	 {  
	     ADC6Exists = 1;
	     ADC6Resolution = ReadEepromInt(54);
  	     SampleRate6 = ReadEepromInt(56);
	     }
	 else
	     {ADC6Exists=0;}
	 
	 if(EEPROM_read(77)==0x01)	 {   
	     ADC7Exists = 1;
	     ADC7Resolution = ReadEepromInt(58);
  	     SampleRate7 = ReadEepromInt(60);
	     }
	 else
	     {ADC7Exists = 0;}
	 
	 // these are for the digital channels. 
	 //Checking if digital channel exists and if yes load sample its sapmle rate
	 //No need for resolution since digital is only 1 or 0 
	 //Look for "EEPROM map.doc" for details of these locations	 
	 if(EEPROM_read(80)==0x01){DigiSampleRate0=ReadEepromInt(100);Digi0Exists=1;}
	 else
	     {Digi0Exists=0;}
	 
	 if(EEPROM_read(81)==0x01){DigiSampleRate1=ReadEepromInt(102);Digi1Exists=1;}
	 else
	     {Digi1Exists=0;}
	 
	 if(EEPROM_read(82)==0x01){DigiSampleRate2=ReadEepromInt(104);Digi2Exists=1;}
	 else
	     {Digi2Exists=0;}
	 
	 if(EEPROM_read(83)==0x01){DigiSampleRate3=ReadEepromInt(106);Digi3Exists=1;}
	 else
	     {Digi3Exists=0;}
	 
	 if(EEPROM_read(84)==0x01){DigiSampleRate4=ReadEepromInt(108);Digi4Exists=1;}
	 else
	     {Digi4Exists=0;}
	 
	 if(EEPROM_read(85)==0x01){DigiSampleRate5=ReadEepromInt(110);Digi5Exists=1;}
	 else
	     {Digi5Exists=0;}
	 
	 if(EEPROM_read(86)==0x01){DigiSampleRate6=ReadEepromInt(112);Digi6Exists=1;}
	 else
	     {Digi6Exists=0;}
	 
	 if(EEPROM_read(87)==0x01){DigiSampleRate7=ReadEepromInt(114);Digi7Exists=1;}
	 else
	     {Digi7Exists=0;}
	 
	 if(EEPROM_read(88)==0x01){DigiSampleRate8=ReadEepromInt(116);Digi8Exists=1;}
	 else
	     {Digi8Exists=0;}
	 
	 if(EEPROM_read(89)==0x01){DigiSampleRate9=ReadEepromInt(118);Digi9Exists=1;}
	 else
	     {Digi9Exists=0;}
	 
	 if(EEPROM_read(90)==0x01){DigiSampleRate10=ReadEepromInt(120);Digi10Exists=1;}
	 else
	     {Digi10Exists=0;}
	 
	 if(EEPROM_read(91)==0x01){DigiSampleRate11=ReadEepromInt(122);Digi11Exists=1;}
	 else
	     {Digi11Exists=0;}
	 
	 if(EEPROM_read(92)==0x01){DigiSampleRate12=ReadEepromInt(124);Digi12Exists=1;}
	 else
	     {Digi12Exists=0;}
	 
	 if(EEPROM_read(93)==0x01){DigiSampleRate13=ReadEepromInt(126);Digi13Exists=1;}
	 else
	     {Digi13Exists=0;}
	 
	 if(EEPROM_read(94)==0x01){DigiSampleRate14=ReadEepromInt(128);Digi14Exists=1;}
	 else
	     {Digi14Exists=0;} 
}

// Sets end of sampling condition.
//   1 For GUI
//   2 is for digital pin High
//   3 is for analog threshold
//   4 is for digital pin Low
void SetEndCondition()
{
     EndCondition=ScanInt();
	 if (EndCondition==2)	 {
	  	  StopPort = ScanInt();
	  	  StopPin = ScanInt();
	      }
	 if (EndCondition==4)	 {
	  	  StopPort = ScanInt();
	  	  StopPin = ScanInt();
	      }
	 else if (EndCondition==3)	 {
		 StopThresholdPin = ScanInt();
		 StopThresholdValue = ScanInt();
	     }
	 
	 WriteEepromInt(12,EndCondition );
     WriteEepromInt(22,StopPort );
	 WriteEepromInt(24,StopPin );
	 WriteEepromInt(26,StopThresholdValue );
	 WriteEepromInt(28,StopThresholdPin );
	 
	 BlinkGreen(2); 
}

//Sets start of sampling condition.
// 1 For GUI
//2 is for digital pin High
//3 is for analog threshold
//4 is for digital pin Low
void SetStartCondition()
{	  
     RED_LED(ON);
	 startCondition = ScanInt();
	  	 
	 if (startCondition==2)	 {
	     StartPort = ScanInt();
	  	 StartPin = ScanInt();
	     }
	 else if (startCondition==3)	 {
	     StartThresholdPin = ScanInt();
	     StartThresholdValue = ScanInt(); 	  
	     }
	 else if (startCondition==4)	 {
	     StartPort = ScanInt();
	  	 StartPin = ScanInt();
	     }
 
	 WriteEepromInt(10,startCondition ); 
	 WriteEepromInt(14,StartPort );
	 WriteEepromInt(16,StartPin );
	 WriteEepromInt(18,StartThresholdValue );
	 WriteEepromInt(20,StartThresholdPin );	 
}

void Inactivate_All_Channels()
{
 	 ADC0Exists=0;	EEPROM_write(70,0x00); 
	 ADC1Exists=0;  EEPROM_write(71,0x00);
	 ADC2Exists=0;	EEPROM_write(72,0x00);
	 ADC3Exists=0;	EEPROM_write(73,0x00);
	 ADC4Exists=0;	EEPROM_write(74,0x00);
	 ADC5Exists=0;	EEPROM_write(75,0x00);
	 ADC6Exists=0;	EEPROM_write(76,0x00);
	 ADC7Exists=0;	EEPROM_write(77,0x00);
	 
	 Digi0Exists=0;  EEPROM_write(80,0x00);
	 Digi1Exists=0;  EEPROM_write(81,0x00);
	 Digi2Exists=0;  EEPROM_write(82,0x00);
	 Digi3Exists=0;  EEPROM_write(83,0x00);
	 Digi4Exists=0;  EEPROM_write(84,0x00);
	 Digi5Exists=0;  EEPROM_write(85,0x00);
	 Digi6Exists=0;  EEPROM_write(86,0x00);
	 Digi7Exists=0;  EEPROM_write(87,0x00);
	 Digi8Exists=0;  EEPROM_write(88,0x00);
	 Digi9Exists=0;  EEPROM_write(89,0x00);
	 Digi10Exists=0; EEPROM_write(90,0x00);
	 Digi11Exists=0; EEPROM_write(91,0x00);
	 Digi12Exists=0; EEPROM_write(92,0x00);
	 Digi13Exists=0; EEPROM_write(93,0x00);
	 Digi14Exists=0; EEPROM_write(94,0x00);	 
}

void Activate_Channel()
{
 	 int Channel;
	 Channel = ScanInt();
	 
	 if(Channel==0) {ADC0Exists=1; EEPROM_write(70,0x01);}
	 if(Channel==1) {ADC1Exists=1; EEPROM_write(71,0x01);}
	 if(Channel==2) {ADC2Exists=1; EEPROM_write(72,0x01);}
	 if(Channel==3) {ADC3Exists=1; EEPROM_write(73,0x01);}
	 if(Channel==4) {ADC4Exists=1; EEPROM_write(74,0x01);}
	 if(Channel==5) {ADC5Exists=1; EEPROM_write(75,0x01);}
	 if(Channel==6) {ADC6Exists=1; EEPROM_write(76,0x01);}
	 if(Channel==7) {ADC7Exists=1; EEPROM_write(77,0x01);}
	 
	 if(Channel==8) {Digi0Exists=1;  EEPROM_write(80,0x01);}
	 if(Channel==9) {Digi1Exists=1;  EEPROM_write(81,0x01);}
	 if(Channel==10) {Digi2Exists=1;  EEPROM_write(82,0x01);}
	 if(Channel==11) {Digi3Exists=1;  EEPROM_write(83,0x01);}
	 if(Channel==12) {Digi4Exists=1;  EEPROM_write(84,0x01);}
	 if(Channel==13) {Digi5Exists=1;  EEPROM_write(85,0x01);}
	 if(Channel==14) {Digi6Exists=1;  EEPROM_write(86,0x01);}
	 if(Channel==15) {Digi7Exists=1;  EEPROM_write(87,0x01);}
	 if(Channel==16) {Digi8Exists=1;  EEPROM_write(88,0x01);}
	 if(Channel==17) {Digi9Exists=1;  EEPROM_write(89,0x01);}
	 if(Channel==18) {Digi10Exists=1; EEPROM_write(90,0x01);}
	 if(Channel==19) {Digi11Exists=1; EEPROM_write(91,0x01);}
	 if(Channel==20) {Digi12Exists=1; EEPROM_write(92,0x01);}
	 if(Channel==21) {Digi13Exists=1; EEPROM_write(93,0x01);}
	 if(Channel==22) {Digi14Exists=1; EEPROM_write(94,0x01);}
}

void AssignConfigToChannel(int resolution,int sampleRate,int pinType,int pinNumber)
{	 
	 if (pinType==0) //type 0 is for analog pins ( ADC's)
	 {
	  	if (pinNumber==0)  {
		    ADC0Resolution=resolution;
		    SampleRate0 =sampleRate;
		    WriteEepromInt(30,ADC0Resolution);//+2 for each address 
		    WriteEepromInt(32,SampleRate0);   //because int takes 2 mem positions
		    EEPROM_write(70,0x01);
		    ADC0Exists=1;
		    }
		else if (pinNumber==1)	{
		    ADC1Resolution=resolution;
		    SampleRate1 =sampleRate;
		    WriteEepromInt(34,ADC1Resolution);
		    WriteEepromInt(36,SampleRate1);
		    EEPROM_write(71,0x01);
		    ADC1Exists=1;
		    }
		else if (pinNumber==2)	{
		    ADC2Resolution=resolution;
		    SampleRate2 =sampleRate;
		    WriteEepromInt(38,ADC2Resolution);
		    WriteEepromInt(40,SampleRate2);
		    EEPROM_write(72,0x01);
		    ADC2Exists=1;
		    }
		else if (pinNumber==3)  {
		    ADC3Resolution=resolution;
		    SampleRate3 =sampleRate;
		    WriteEepromInt(42,ADC3Resolution);
		    WriteEepromInt(44,SampleRate3);
		    EEPROM_write(73,0x01);
		    ADC3Exists=1;
		    }
		else if (pinNumber==4)	{
		    ADC4Resolution=resolution;
   	        SampleRate4 =sampleRate;
		    WriteEepromInt(46,ADC4Resolution);
		    WriteEepromInt(48,SampleRate4);
		    EEPROM_write(74,0x01);
		    ADC4Exists=1;
		    }
		else if (pinNumber==5)	{
		    ADC5Resolution=resolution;
		    SampleRate5 =sampleRate;
		    WriteEepromInt(50,ADC5Resolution);
		    WriteEepromInt(52,SampleRate5);
		    EEPROM_write(75,0x01);
		    ADC5Exists=1;
		    }
		else if (pinNumber==6)	{
		    ADC6Resolution=resolution;
		    SampleRate6 =sampleRate;
		    WriteEepromInt(54,ADC6Resolution);
		    WriteEepromInt(56,SampleRate6);
		    EEPROM_write(76,0x01);
		    ADC6Exists=1;
		    }
		else if (pinNumber==7)	{
		    ADC7Resolution=resolution;
		    SampleRate7 =sampleRate;
		    WriteEepromInt(58,ADC7Resolution);
		    WriteEepromInt(60,SampleRate7);
		    EEPROM_write(77,0x01);
		    ADC7Exists=1;
		    }
	    }
	 else if (pinType==1) // Type 1 is for digital pins
	 { 	  
		  //Your code for digital pins goes here.
		  if (pinNumber==0)
		      {DigiSampleRate0=sampleRate;Digi0Exists=1; WriteEepromInt(100,DigiSampleRate0); EEPROM_write(80,0x01);}
		  else if (pinNumber==1)
		      {DigiSampleRate1=sampleRate;Digi1Exists=1; WriteEepromInt(102,DigiSampleRate1); EEPROM_write(81,0x01);}
		  else if (pinNumber==2)
		      {DigiSampleRate2=sampleRate;Digi2Exists=1; WriteEepromInt(104,DigiSampleRate2); EEPROM_write(82,0x01);}
		  else if (pinNumber==3)
		      {DigiSampleRate3=sampleRate;Digi3Exists=1; WriteEepromInt(106,DigiSampleRate3); EEPROM_write(83,0x01);}
		  else if (pinNumber==4)
		      {DigiSampleRate4=sampleRate;Digi4Exists=1; WriteEepromInt(108,DigiSampleRate4); EEPROM_write(84,0x01);}
		  else if (pinNumber==5)
		      {DigiSampleRate5=sampleRate;Digi5Exists=1; WriteEepromInt(110,DigiSampleRate5); EEPROM_write(85,0x01);}
		  else if (pinNumber==6)
		      {DigiSampleRate6=sampleRate;Digi6Exists=1; WriteEepromInt(112,DigiSampleRate6); EEPROM_write(86,0x01);}
		  else if (pinNumber==7)
		      {DigiSampleRate7=sampleRate;Digi7Exists=1; WriteEepromInt(114,DigiSampleRate7); EEPROM_write(87,0x01);}
		  else if (pinNumber==8)
		      {DigiSampleRate8=sampleRate;Digi8Exists=1; WriteEepromInt(116,DigiSampleRate8); EEPROM_write(88,0x01);}
		  else if (pinNumber==9)
		      {DigiSampleRate9=sampleRate;Digi9Exists=1; WriteEepromInt(118,DigiSampleRate9); EEPROM_write(89,0x01);}
		  else if (pinNumber==10)
		      {DigiSampleRate10=sampleRate;Digi10Exists=1; WriteEepromInt(120,DigiSampleRate10); EEPROM_write(90,0x01);}
		  else if (pinNumber==11)
		      {DigiSampleRate11=sampleRate;Digi11Exists=1; WriteEepromInt(122,DigiSampleRate11); EEPROM_write(91,0x01);}	  
		  else if (pinNumber==12)
		      {DigiSampleRate12=sampleRate; Digi12Exists=1; WriteEepromInt(124,DigiSampleRate12); EEPROM_write(92,0x01);}	  
		  else if (pinNumber==13)
		      {DigiSampleRate13=sampleRate;Digi13Exists=1; WriteEepromInt(126,DigiSampleRate13); EEPROM_write(93,0x01);}
		  else if (pinNumber==14)
		      {DigiSampleRate14=sampleRate;Digi14Exists=1; WriteEepromInt(128,DigiSampleRate14); EEPROM_write(94,0x01);}		   
	      }	  
}

// Configure Channel characteristics from host
void channelConfigReceiver()
{
 	 int resolution;
	 int sampleRate; 
	 int pinType;
	 int pinNumber;
	
	 RED_LED(ON);
	 GREEN_LED(ON);
	 resolution=ScanInt();
	 sampleRate=ScanInt();
	 pinType=ScanInt();//0 for analog pins. and 1 for digital pins 
	  
	 pinNumber=ScanInt();//0 for ADC0, 1 for ADC1, etc.
	 
	 AssignConfigToChannel(resolution,sampleRate,pinType,pinNumber);
	 
	 Delaymsec(750);
	 RED_LED(OFF);
	 GREEN_LED(OFF);
}


void checkStopPinHigh()
{	 
	 if (!(TestBit(StopPort,StopPin)))	{
	     StopPinFlag=1;
	     }	
}

void checkStopPinLow()
{	 
	 if ((TestBit(StopPort,StopPin))) {
	     StopPinFlag=1;
	     }	
}

void checkStartPinHigh()
{	 
	 if (!(TestBit(StopPort,StopPin))) {
	     dataAcquisition();
	     }	
}

void checkStartPinLow()
{	 
	 if ((TestBit(StopPort,StopPin))) {
	     dataAcquisition();
	     }	
}

void checkStartThreshold()
{
	 if(StartThresholdPin==0)
	 {if (analogValue0>StartThresholdValue){StartThresholdFlag=1;}
	 }
 	 else if(StartThresholdPin==1)
	 {if (analogValue1>StartThresholdValue){StartThresholdFlag=1;}
	 }
	 else if(StartThresholdPin==2)
	 {if (analogValue2>StartThresholdValue){StartThresholdFlag=1;}
	 }
 	 else if(StartThresholdPin==3)
	 {if (analogValue3>StartThresholdValue){StartThresholdFlag=1;}
	 }	 
	 else if(StartThresholdPin==4)
	 {if (analogValue4>StartThresholdValue){StartThresholdFlag=1;}
	 }
 	 else if(StartThresholdPin==5)
	 {if (analogValue5>StartThresholdValue){StartThresholdFlag=1;}
	 }
	 else if(StartThresholdPin==6)
	 {if (analogValue6>StartThresholdValue){StartThresholdFlag=1;}
	 }
	 else if(StartThresholdPin==7)
	 {if (analogValue7>StartThresholdValue){StartThresholdFlag=1;}
	 }
	 if (StartThresholdFlag==1)
	 {
	  	StartThresholdFlag=0;	 
	  	dataAcquisition(); 	
	 } 
}

void checkStopThreshold()
{	 	 
	 if(StopThresholdPin==0)
	 {if (analogValue0>StopThresholdValue){StopThresholdFlag=1;}
	 }
 	 else if(StopThresholdPin==1)
	 {if (analogValue1>StopThresholdValue){StopThresholdFlag=1;}
	 }
	 else if(StopThresholdPin==2)
	 {if (analogValue2>StopThresholdValue){StopThresholdFlag=1;}
	 }
 	 else if(StopThresholdPin==3)
	 {if (analogValue3>StopThresholdValue){StopThresholdFlag=1;}
	 }	 
	 else if(StopThresholdPin==4)
	 {if (analogValue4>StopThresholdValue){StopThresholdFlag=1;}
	 }
 	 else if(StopThresholdPin==5)
	 {if (analogValue5>StopThresholdValue){StopThresholdFlag=1;}
	 }
	 else if(StopThresholdPin==6)
	 {if (analogValue6>StopThresholdValue){StopThresholdFlag=1;}
	 }
	 else if(StopThresholdPin==7)
	 {if (analogValue7>StopThresholdValue){StopThresholdFlag=1;}
	 }
}

void Scheduler()
{
 	 if(startCondition==3)	 {
	  	 checkStartThreshold();
	     }
	 else if(startCondition==2) {
	   	 checkStartPinHigh();
	     } 
	 else if(startCondition==4) {
	   	 checkStartPinLow();
	     } 
}


unsigned int  ScanInt()
{
//maximum value is 32767 (this number probably change on a different machine)
 	 unsigned int  integer,i;
	 
	 integer = 0;
	 putcharr((instrg[0]=getcharr()));
 	 while((instrg[i]!=CR)&&(instrg[i]!=' ')) {
	 	 i++;
	     putcharr((instrg[i]=getcharr()));
		 }
	 
	 integer=strtol(instrg,NULL,10);
	 	 
	 //printf ("here is your int: %d ",integer);
	 return integer;
}

//void dataAcquisition()
//{
//}


void Check_Acq_Loop_led()
{
	// Check toggle condition of Green Led
	if(mseccountdown1==0) {
	    mseccountdown1 = acqloopsetting;
		if(acqtoggle) {
		    GREEN_LED(OFF);
			acqtoggle = 0;
			}
		else {
		    GREEN_LED(ON);
			acqtoggle = 1;
			}
	    }
}

void StartPageWrite()
{
 	 int i;
	 union {
	    unsigned char tempbuf[2];
		int outiint;
		} u;
	 
	 //printf("start page write\n\r");
	 
	 // Serial Flash CS low
	 FlashChipSelect(LOW);	
	  
	 //  Select Buffer to write too
	 if(bufferidptr==1)  SPI_MasterTransmit(SF_WRITE_BUFFER1);
	 else  SPI_MasterTransmit(SF_WRITE_BUFFER2);
	 
	 //printf("ptr=%d ",bufferidptr);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(0);
	 //ConvertPower2FlashAddress(flashaddress);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 pagemode = OLD;
	 SPI_MasterTransmit('y');
	 //SPI_MasterTransmit(data2log);
	 SPI_MasterTransmit(10);
	 //SPI_MasterTransmit(SensorRangeByte());
	 SPI_MasterTransmit(0xff);
	 //u.outiint = logmsec;
	 u.outiint = 10;
	 SPI_MasterTransmit(u.tempbuf[1]);
	 SPI_MasterTransmit(u.tempbuf[0]);
	 SPI_MasterTransmit(days);
	 SPI_MasterTransmit(hours); 
	 SPI_MasterTransmit(minutes);
	 SPI_MasterTransmit(seconds);
	 SPI_MasterTransmit(rtcclockcnt);
	 
//	 printf("leave start page write\n\r");
}

void FinishPageWrite(int pagecnt)
{
 	 int i;
	 
	 //printf("finish page write\n\r");
	 
	 // Close off the prior state so new state takes effect
	 FlashChipSelect(HIGH);
	 
	 // Store flash buffer to flashpage
	 FlashChipSelect(LOW);
	  
	 // Transfer buffer n to flash memory
	 if(bufferidptr==1) SPI_MasterTransmit(SF_BUF1_TO_FLASH_NOERASE);
	 else SPI_MasterTransmit(SF_BUF2_TO_FLASH_NOERASE);
	 
	 // Convert address to 24 bit address
	 ConvertFlashAddress(flashaddress);
	 //ConvertPower2FlashAddress(flashaddress);
	 for(i=0; i<3; i++)  SPI_MasterTransmit(FlashAddress[i]);
	 
	 // Progamming the flash can take up to 40msec so make sure the data 
	 // logging rate is not to high
	 FlashChipSelect(HIGH);
	 		
	 // Flip the bufferid ptr
	 if(bufferidptr==1) bufferidptr = 2;
	 else bufferidptr = 1;
	 
	 // See if flash is full
 	 if(flashtype0==AT45DB081ID)  {
	     flashaddress += 256;
		 if(++pagecnt>4096) loggingmode = HALT;
		 }
	 else if(flashtype0==AT45DB642ID)  {
	     flashaddress += 1024;
	 	 if(++pagecnt>8192)  loggingmode = HALT;
		 }
//	 printf("leave finish page write\n\r");
}

int CalculateBufferLength(int headerlength)
{
 	int buflength;
	
	if(flashtype0==AT45DB081ID) buflength = 256-headerlength;
	else if(flashtype0==AT45DB642ID) buflength = 1024-headerlength;
	return buflength;
}

int CheckPageWrite(int buflength,int pagecnt,int headerlength)
{		
    buflength -= 3;
	if(buflength<=0) {
		FinishPageWrite(pagecnt);
		StartPageWrite();
		buflength = CalculateBufferLength(headerlength);
		}	
	return buflength;	    		     
}


// Main data acquisition loop
//   TBD - log to serial flash
//       - time stamp data
//
// At 57600 - 5236 bytes/second transferred
// For analog 3bytes/sample = 1740 samples/second
// Raw cycle rate through first level compares is 43,000/second
void dataAcquisition()
{ 
	int buflength,headerlength,pagecnt;
    union u_tag {
	    int iivalue;
		unsigned char intchar[2];
		} u; 
	unsigned char inchar;	
	
	mseccountdown1 = acqloopsetting;
	pagecnt = 0;
	flashaddress = 0;
	
	while((StopThresholdFlag==0)&&(StopPinFlag==0)) {
	 
	    inchar = getcharnw();
		
 	    // Setup beginning information for flash page
	    // First byte is always 'y' character indicating flash page is full
	    // Next the sample rate is written followed by the current time
	    // so retrieved data can be properly time stamped.
		if((localStorage==1)&&(pagemode==NEW))  {
		    StartPageWrite();
	        headerlength = 10;
			
			buflength = CalculateBufferLength(headerlength);	
			}

		// Slow data acq rate to 1msecond
		while(onemsecond==1) ;
		
		//dataacqloopcnt++;
		
		if((inchar!=0)&&(inchar!='&')) break;
		
      	Check_Acq_Loop_led();
		
			if (ADC0Exists==1)	{														
    	  		if (milSecTimer0==0) {
    			   	 if(localStorage==0) {
            		     putchar(0x00);	
            		     u.iivalue = analogValue0;
            			 putchar(u.intchar[0]);
            			 putchar(u.intchar[1]);
    				     }
    				 else if (localStorage==1)	 {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x00);
            		     u.iivalue = analogValue0;
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
    				     }
        			 milSecTimer0 = SampleRate0;
    			     }
    			 else{milSecTimer0--;}
			     }
			 
			 if (ADC1Exists==1)	 {
    			 if (milSecTimer1==0) {    
    				 if(localStorage==0) {
            		     putchar(0x01);	
            		     u.iivalue = analogValue1;
            			 putchar(u.intchar[0]);
            			 putchar(u.intchar[1]);
    				     }
    				 else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x01);
            		     u.iivalue = analogValue1;
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
    				     }
        			 milSecTimer1 = SampleRate1;
    			     }
    			 else{milSecTimer1--;} 
			     }
			 	 
			 if (ADC2Exists==1)	 {
    			 if (milSecTimer2==0) {
    			 	 if(localStorage==0) {				  
            		     putchar(0x02);	
            		     u.iivalue = analogValue2;
            			 putchar(u.intchar[0]);
            			 putchar(u.intchar[1]);
    				     }
    				 else if(localStorage==1)   {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x02);
            		     u.iivalue = analogValue2;
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
    				     }
        			 milSecTimer2 = SampleRate2;
    			     }
    			 else{milSecTimer2--;}
			     }
			 	 
			 if (ADC3Exists==1)	 {
    			 if (milSecTimer3==0) {
    			 	 if(localStorage==0) {
            		     putchar(0x03);	
            		     u.iivalue = analogValue3;
            			 putchar(u.intchar[0]);
            			 putchar(u.intchar[1]);
    				     }
    				 else if(localStorage==1)  {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x03);
            		     u.iivalue = analogValue3;
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
    				     }
        			 milSecTimer3 = SampleRate3;
    			     }
    			 else{milSecTimer3--;}
    		     }	 
			 
			if (ADC4Exists==1)	{
    	  		if (milSecTimer4==0) {
    			 	 if(localStorage==0) {
            		     putchar(0x04);	
            		     u.iivalue = analogValue4;
            			 putchar(u.intchar[0]);
            			 putchar(u.intchar[1]);  
						 }
    				 else if(localStorage==1)  {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x04);
            		     u.iivalue = analogValue4;
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
					     }
        			 milSecTimer4 = SampleRate4;
    			     }
    			 else{milSecTimer4--;}
			     }	 
    			 
			 if (ADC5Exists==1)	 {
    			 if (milSecTimer5==0) {
    			  	 if(localStorage==0) {
            			 putchar(0x05);	
            		     u.iivalue = analogValue5;
            			 putchar(u.intchar[0]);
            		     putchar(u.intchar[1]);
    				     }
    				 else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x05);
            		     u.iivalue = analogValue5;
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
					     }
        		     milSecTimer5 = SampleRate5;	 
    			     }
    			 else{milSecTimer5--;}
    		     }	 			
				
			 if (ADC6Exists==1) {			 
    			 if (milSecTimer6==0) {
    			  	 if(localStorage==0) {
            			 putchar(0x06);	
            		     u.iivalue = analogValue6;
            			 putchar(u.intchar[0]);
            			 putchar(u.intchar[1]);
    				     }
    				 else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x06);
            		     u.iivalue = analogValue6;
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
					     }
        			 milSecTimer6 = SampleRate6;
    			     }
    			 else{milSecTimer6--;}
			     }

			 if (ADC7Exists==1) {
    			 if (milSecTimer7==0) {
       			  	 if(localStorage==0) {
            			 putchar(0x07);	
            		     u.iivalue = analogValue7;
            			 putchar(u.intchar[0]);
            			 putchar(u.intchar[1]);
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x07);
            		     u.iivalue = analogValue7;
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
					    }	 
        			milSecTimer7 = SampleRate7;
    			    }
    			else{milSecTimer7--;}
			    } 					     
			 
			 //For B1
			 if (Digi0Exists==1) {
    			 if (DigiMilSecTimer0==0) {
    			  	 if(localStorage==0) {
            			putchar(0x08);						 
						if(TestBit(PORTB,PORTB1))
						    {putchar(0x01);putchar(0x00);}
						else{putchar(0x00); putchar(0x00);}     
    				    }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x08);
						 SPI_MasterTransmit(u.intchar[0]);
						 SPI_MasterTransmit(u.intchar[1]);
					    }	 
        			DigiMilSecTimer0=DigiSampleRate0;
    			    }
    			 else{DigiMilSecTimer0--;}
			     } 			
			 
			 //For B2:channel 9
			  if (Digi1Exists==1) {
    			 if (DigiMilSecTimer1==0) {		 
    			  	 if(localStorage==0) {
            			 putchar(0x09);	
						 if(TestBit(PORTB,PORTB2))
						 { putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}     		     
    				     }
    				 else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x09);
						 if(TestBit(PORTB,PORTB2))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					    } 
        			 DigiMilSecTimer1=DigiSampleRate1;
    			     }
    			 else{DigiMilSecTimer1--;}
			     } 					      		
			 
			 //For B4:channel 11
			  if (Digi3Exists==1) {
    			 if (DigiMilSecTimer3==0) {	 
    			  	 if(localStorage==0) {
            			 putchar(0x0B);//11	
						 if(TestBit(PORTB,PORTB4))
						     {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x0B);
						 if(TestBit(PORTB,PORTB4))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					    }				 
        			DigiMilSecTimer3=DigiSampleRate3;
    			    }
    			 else{DigiMilSecTimer3--;}
			     } 					      		

			 //For B5:channel 12
			  if (Digi4Exists==1) {
    			 if (DigiMilSecTimer4==0) {    			 
    			  	 if(localStorage==0) {
            			 putchar(0x0C);//12	
						 if(TestBit(PORTB,PORTB5))
						     {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}    
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x0C);
						 if(TestBit(PORTB,PORTB5))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					    } 
        			DigiMilSecTimer4=DigiSampleRate4;
    			    }
    			 else{DigiMilSecTimer4--;}
			     } 					      		
			 
			  //For B6:channel 13
			  if (Digi5Exists==1) {
    			 if (DigiMilSecTimer5==0) { 			 
    			  	 if(localStorage==0) {
            			 putchar(0x0D);//13	
						 if(TestBit(PORTB,PORTB6))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x0D);
						 if(TestBit(PORTB,PORTB6))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					    }	 
        			DigiMilSecTimer5=DigiSampleRate5;
    			    }
    			 else{DigiMilSecTimer5--;}
			     } 					      		

			 //For B7:channel 14	 
			  if (Digi6Exists==1) {
    			 if (DigiMilSecTimer6==0) {   			 
    			  	 if(localStorage==0) {
            			 putchar(0x0E);//14	
						 if(TestBit(PORTB,PORTB7))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}            		     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x0E);
						 if(TestBit(PORTB,PORTB7))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					    }    					 
        			DigiMilSecTimer6=DigiSampleRate6;
    			    }
    			 else{DigiMilSecTimer6--;}
			     } 					      		
			 
		  //For C0:channel 15
			  if (Digi7Exists==1) {
    			 if (DigiMilSecTimer7==0) {   			 
    			  	 if(localStorage==0) {
            			 putchar(0x0F);//15	
						 if(TestBit(PORTC,PORTC0))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}            		     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x0F);
						 if(TestBit(PORTC,PORTC0))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					     }	 
        			DigiMilSecTimer7=DigiSampleRate7;
    			    }
    			 else{DigiMilSecTimer7--;}
			     } 					      		
			 
			 //For C1:channel 16
			  if (Digi8Exists==1) {
    			 if (DigiMilSecTimer8==0) {  			 
    			  	 if(localStorage==0) {
            			 putchar(0x10);//16	
						 if(TestBit(PORTC,PORTC1))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x10);
						 if(TestBit(PORTC,PORTC1))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					    } 
        			DigiMilSecTimer8=DigiSampleRate8;
    			    }
    			 else{DigiMilSecTimer8--;}
			     } 					      		
			 
			 //For D0:channel 17
			  if (Digi9Exists==1) {
    			 if (DigiMilSecTimer9==0) {    			 
    			  	 if(localStorage==0) {
            			 putchar(0x11);	//17
						 if(TestBit(PORTD,PORTD0))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}            		     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x11);
						 if(TestBit(PORTD,PORTD0))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					    }   					 
        			DigiMilSecTimer9=DigiSampleRate9;
    			    }
    			 else{DigiMilSecTimer9--;}
			     } 					      		
			 
			 //For D1:channel 18
			  if (Digi10Exists==1) {
    			 if (DigiMilSecTimer10==0)	 {    			 
    			  	 if(localStorage==0) {
            			 putchar(0x12);//18	
						 if(TestBit(PORTD,PORTD1))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}            		     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x12);
						 if(TestBit(PORTD,PORTD1))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					     }   					 
        			DigiMilSecTimer10=DigiSampleRate10;
    			    }
    			 else{DigiMilSecTimer10--;}
			     } 					      		
			 
			 //For D2:channel 19
			 if (Digi11Exists==1) {
    			 if (DigiMilSecTimer11==0) {    			 
    			  	 if(localStorage==0) {
            			 putchar(0x13);	//19
						 if(TestBit(PORTD,PORTD2))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}            		     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x13);
						 if(TestBit(PORTD,PORTD2))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					     }    					 
        			DigiMilSecTimer11=DigiSampleRate11;
    			    }
    			 else{DigiMilSecTimer11--;}
			     } 					      		
			 
			 //For D3:channel 20
			 if (Digi12Exists==1) {
    			 if (DigiMilSecTimer12==0) {    			 
    			  	 if(localStorage==0) {
            			 putchar(0x14);	//20
						 if(TestBit(PORTD,PORTD3))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}            		     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x14);
						 if(TestBit(PORTD,PORTD3))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					     }   					 
        			DigiMilSecTimer12=DigiSampleRate12;
    			    }
    			 else{DigiMilSecTimer12--;}
			     } 					      		
			 
			 //For D6:channel 21
			 if (Digi13Exists==1) {
    			 if (DigiMilSecTimer13==0) {    			 
    			  	 if(localStorage==0) {
            			 putchar(0x15);	//21
						 if(TestBit(PORTD,PORTD6))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}            		     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x15);
						 if(TestBit(PORTD,PORTD6))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					     }				 
        			DigiMilSecTimer13=DigiSampleRate13;
    			    }
    			 else{DigiMilSecTimer13--;}
			     } 					      		
			 
			 //For D7:channel 22
			 if (Digi14Exists==1) {
    			 if (DigiMilSecTimer14==0) {   			 
    			  	 if(localStorage==0) {
            			 putchar(0x16);	//22
						 if(TestBit(PORTD,PORTD7))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}	     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x16);
						 if(TestBit(PORTD,PORTD7))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					     }   					 
        			DigiMilSecTimer14=DigiSampleRate14;
    			    }
    			 else{DigiMilSecTimer14--;}
			     } 					      		

			 //For B3:channel 10 
			 if (Digi2Exists==1) {
    			 if (DigiMilSecTimer2==0) {   			 
    			  	 if(localStorage==0) {
	// Notice:Putchar(0x0A) will send out 10 and 13 as opposed to just 10. (0X0A in hex is 10 in base 10)
	//The reason is in ASCII 10 is \n and 13 is \r. Everytime you 'Putchar' \n it will send out \n (or 10) and \r (or 13) 
	//at the same time.
	// This extra 13 would have caused synch. problem. Hence channel 10 is encoded as channel 23(0X17) 
            			 putcharr(0x17);//10	
						 if(TestBit(PORTB,PORTB3))
						 {putchar(0x01);putchar(0x00);}
						 else{putchar(0x00);putchar(0x00);}          		     
    				     }
    				else if(localStorage==1) {
					     if((buflength=CheckPageWrite(buflength,pagecnt,headerlength))==-1) goto FINISHED;
    				     SPI_MasterTransmit(0x17);
						 if(TestBit(PORTB,PORTB3))
						 { SPI_MasterTransmit(0x01);SPI_MasterTransmit(0x00);}
						 else{SPI_MasterTransmit(0x00);SPI_MasterTransmit(0x00);}     		     
					     }  					 
        		    DigiMilSecTimer2=DigiSampleRate2;
    			    }
    			 else{DigiMilSecTimer2--;}
			     } 					      		
			 
			if(EndCondition==3)	{
	            checkStopThreshold();
			    }
			else if(EndCondition==2) {
			   checkStopPinHigh();
			   }
			else if(EndCondition==4) {
			   checkStopPinLow();
			   }
			
			// Set semiphore to slow data acquisition rate to 1msec
			onemsecond = 1;
	}
	
FINISHED:
    if(localStorage==1) FlashChipSelect(HIGH);  

	StopThresholdFlag=0;
	StopPinFlag=0;
	
	BlinkRed(2);
	BlinkGreen(2);	
}

void Inactivate_Analog()
{
 	 ADC0Exists=0;	EEPROM_write(70,0x00); 
	 ADC1Exists=0;  EEPROM_write(71,0x00);
	 ADC2Exists=0;	EEPROM_write(72,0x00);
	 ADC3Exists=0;	EEPROM_write(73,0x00);
	 ADC4Exists=0;	EEPROM_write(74,0x00);
	 ADC5Exists=0;	EEPROM_write(75,0x00);
	 ADC6Exists=0;	EEPROM_write(76,0x00);
	 ADC7Exists=0;	EEPROM_write(77,0x00);
}

// Retrieved logged data from Serial Flash
// Retreive logged flash data until no more left.
// Data structure is as follows:
//    Data present in page - 'y' else anything else
//    Time stamp - day,hour,min,second,msec 
void GetLoggedData()
{
     unsigned char notfinished,msec,second,min,hour,day,loggeddata;
 	 unsigned char byte;
	 unsigned char inchar,sensorrange,inbyte;
	 int bytesinpage,lograte,pagecnt;
	 int headerlength,buflengthend;
	 float floattime;

	 //  Set PC4 low - Flash CS
	 FlashChipSelect(LOW);  

	 SPI_StartReadFlashPage(0);
	 pagecnt = 1;
	 notfinished = YES;

	 while((notfinished==YES)) {
	 	 inchar = getcharnw();
		 // Pause output until any key hit
		 if(inchar=='p') while((getcharnw()==0)) ;
		 // Halt output and exit data log dump
		 else if(inchar!=0) break;

	     byte = SPI_MasterReceive();

		 //printf("\n\rfirst byte = %x\n\r",byte);

		 // See if first page byte is 'y' - process page if it is
		 if(byte=='y') {
		     loggeddata = SPI_MasterReceive();
			 sensorrange = SPI_MasterReceive();

			 //printf("logged data = %c  combinedbyte= %x\n\r",loggeddata,combinedbyte);

		     // Retrieve log rate
			 lograte = ((int)SPI_MasterReceive())*256 + (int)SPI_MasterReceive();

		     // Retrieve time stamp
			 day = SPI_MasterReceive();
			 hour = SPI_MasterReceive();
			 min = SPI_MasterReceive();
			 second = SPI_MasterReceive();
			 msec = SPI_MasterReceive();
			 
			 // Get the number of bytes in the page
			 bytesinpage = CalculateBufferLength(10);
			 headerlength = 10;

			 floattime = (float) second + ((float) (msec*4))/1000.0;
			 
			 // Output logged data if at least one sensor is present
			 //printf("%ld ",recordcnt++);
			 //printf("%2d:%2d:%2d:%#2.3f",day,hour,min,floattime);
			 floattime += ((float) lograte)/1000.0;
			 if(floattime>60.0) {
			 	floattime -= 60.0;
				min++;
				if(min>59) {
					min = 0;
					hour++;
					if(hour>23) {
						hour = 0;
						day++;
						}
					}
				}
			
			while(bytesinpage--)  {
				inbyte = SPI_MasterReceive();
				putchar(inbyte);
				//printf("%x ",inbyte);
			    }
			
			// Skip the blank bytes at the end of the page
			if(flashtype0==AT45DB081ID) {
				//buflength = 9;
				buflengthend = 256 - headerlength - bytesinpage;
				if(pagecnt++>4096) notfinished = NO;
				}
	 		else if(flashtype0==AT45DB642ID) {
				//buflength = 3;
				buflengthend = 1024 - headerlength - bytesinpage;
				if(pagecnt++>8192) notfinished = NO;
				}
			//for(i=0; i<buflengthend; i++ )  SPI_MasterReceive();			 		 		 
		    }
		 else {
		     //printf("Finished processing pages");
			 notfinished = NO;
			 } 
		 }
     
	 // Set PC4 high - deselect Flash chip
	 FlashChipSelect(HIGH);  
}

void testing()
{
	// SetBit(PORTD,PIND4);
	if(TestBit(PORTD,PORTD4)) {
	    printf("On passed!");
		}

	else if(!TestBit(PORTD,PORTD4)) {
	    printf("OFF passed!");
		}

}
