/**************************************************************

   Program Title:  IMU64X0 Data Logger
   
   Program Name:  IMU64X0_ISRs.c	  Version: 1.00
   
   Author:  SS	  					  Date:  Feb 22, 2013
   
   Compiled with AVR Studio 4.19.
   
   This file contains routines that implement the interrupt service routines 
   for the IMU64X0 Monitor.
   
   Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Machines, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC Robotics, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU64X0_ISRs.c,v $
 * Revision 1.00  2013/02/22 SS
 *    Support for ZigBee added
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
// Include header files for GCC/IAR
#include "avr_compiler.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "IMU64X0_Defines.h"
#include "IMU64X0_Type.h"
#include "IMU64X0_Uart.h"
#include "IMU64X0_Extern.h"

//*********************************************************
//*********************************************************
//     Interrupt handlers
//*********************************************************
//*********************************************************

ISR(USART0_RX_vect)
{
	unsigned char data,tmphead;
	
	data = UCSR0A;
	if((data&0x10)) frameerror = YES;
	if((data&0x08)) dataoverrun = YES;
	
	data = UDR0;   // read the received data
	
	tmphead = (UART_RxHead+1)&UART_RX_BUFFER_MASK;  // calculate buffer index
	UART_RxHead = tmphead;      // store new index
	UART_RxBuf[tmphead] = data; // store received data in buffer
	
	rxcnt++;
	if(rxcnt>UART_RX_BUFFER_SIZE-1) rxcnt = UART_RX_BUFFER_SIZE-2;
	
	totalrxcnt++;
	if(rxcnt>rxcntmax)  rxcntmax = rxcnt;

	// If receive buffer almost full Fire TX interrupt if 
	// transmit buffer empty otherwise alert Tx int routine XOFF must go
	if((rxcnt>(UART_RX_BUFFER_SIZE-30))&&(xoffsent==NO))  {
	    if((UCSR0A&0x20))  UCSR0B |= (1<<UDRIE0);
		else sendxoff = YES;
		}
	
//	if((rxcnt>(UART_RX_BUFFER_SIZE-10))&&(xoffsent==NO)&&(enablexonoff==YES)) {
//	if((rxcnt>(UART_RX_BUFFER_SIZE-10))&&(xoffsent==NO)) {
	// If tx buffer is empty then transmit xoff else signal tx to send xoff
//	    sendxonxoff = YES;
//		xoffsent = YES;
//		UCSRB |= (1<<UDRIE);  // Fire TX interrupt	
//		}
}

ISR(USART0_UDRE_vect)
{
	unsigned char tmptail;

	// Check if all data has been transmitted
	if(UART_TxHead != UART_TxTail)  {
	    // Check chars in rx buffer - if too many - transmit XOFF
		if((rxcnt>(UART_RX_BUFFER_SIZE-34))&&((xoffsent==NO)||(sendxoff==YES))){
		    UDR0 = XOFF;
			xoffcnt++;
			xoffsent = YES;
			sendxoff = NO;
			}
		else if((rxcnt<5)&&(xoffsent==YES)) {
		    UDR0 = XON;
			xoncnt++;
			xoffsent = NO;
		    }
		else {
			 // Calculate buffer index
			 tmptail = (UART_TxTail + 1) & UART_TX_BUFFER_MASK;
			 UART_TxTail = tmptail;     // Store new index
			 UDR0 = UART_TxBuf[tmptail]; // Start transmission
//			 if(UART_TxBuf[tmptail]==XON) {
//			 	  xoffsent = NO;
//				  xonsent = NO;
//				  }
			 }
		}
	else  {  // Check chars in rx buffer - if too many trans XOFF
		if((rxcnt>(UART_RX_BUFFER_SIZE-34))&&(xoffsent==NO)) {
		    UDR0 = XOFF;
			xoffsent = YES;
			xoffcnt++;
			}
		else if((rxcnt<5)&&(xoffsent==YES)) {
		    UDR0 = XON;
			xoncnt++;
			xoffsent = NO;
		    }
		else UCSR0B &= ~(1<<UDRIE0); // Disable UDRE interrupt
		}
}

ISR(USART1_RX_vect)
{
	unsigned char data,tmphead;
	
	data = UCSR1A;
	if((data&0x10)) frameerror = YES;
	if((data&0x08)) dataoverrun = YES;
	
	data = UDR1;   // read the received data
	
	tmphead = (UART_RxHead1+1)&UART_RX_BUFFER_MASK;  // calculate buffer index
	UART_RxHead1 = tmphead;      // store new index
	UART_RxBuf1[tmphead] = data; // store received data in buffer
	
	rxcnt1++;
	if(rxcnt1>UART_RX_BUFFER_SIZE-1) rxcnt1 = UART_RX_BUFFER_SIZE-2;
	
	totalrxcnt1++;
	if(rxcnt1>rxcntmax1)  rxcntmax1 = rxcnt1;

	// If receive buffer almost full Fire TX interrupt if 
	// transmit buffer empty otherwise alert Tx int routine XOFF must go
	if((rxcnt1>(UART_RX_BUFFER_SIZE-30))&&(xoffsent1==NO))  {
	    if((UCSR1A&0x20))  UCSR1B |= (1<<UDRIE1);
		else sendxoff1 = YES;
		}
}

ISR(USART1_UDRE_vect)
{
	unsigned char tmptail;

	// Check if all data has been transmitted
	if(UART_TxHead1 != UART_TxTail1)  {
	    // Check chars in rx buffer - if too many - transmit XOFF
		if((rxcnt1>(UART_RX_BUFFER_SIZE-34))&&((xoffsent1==NO)||(sendxoff1==YES))){
		    UDR1 = XOFF;
			xoffcnt1++;
			xoffsent1 = YES;
			sendxoff1 = NO;
			}
		else if((rxcnt1<5)&&(xoffsent1==YES)) {
		    UDR1 = XON;
			xoncnt1++;
			xoffsent1 = NO;
		    }
		else {
			 // Calculate buffer index
			 tmptail = (UART_TxTail1 + 1) & UART_TX_BUFFER_MASK;
			 UART_TxTail1 = tmptail;     // Store new index
			 UDR1 = UART_TxBuf1[tmptail]; // Start transmission
			 }
		}
	else  {  // Check chars in rx buffer - if too many trans XOFF
		if((rxcnt1>(UART_RX_BUFFER_SIZE-34))&&(xoffsent1==NO)) {
		    UDR1 = XOFF;
			xoffsent1 = YES;
			xoffcnt1++;
			}
		else if((rxcnt1<5)&&(xoffsent1==YES)) {
		    UDR1 = XON;
			xoncnt1++;
			xoffsent1 = NO;
		    }
		else UCSR1B &= ~(1<<UDRIE1); // Disable UDRE interrupt
		}
}

//  Timer0 interrupts every msec - This routines decrements 
//  countdown timers - other routines can control timing to 
//  within 1 msec by monitoring these software timers
ISR(TIMER0_COMPA_vect)
{
 	if(mseccountdown1>0)  mseccountdown1--;
 	if(mseccountdown2>0)  mseccountdown2--;
	if(mseccountdown3>0)  mseccountdown3--;
	if(mseccountdown4>0)  mseccountdown4--;
	if(flashcountdown>0)  flashcountdown--;
	if(bmp085countdown>0)  bmp085countdown--;
}

char not_leap(void)      //check for leap year
{
    if (!(year%100))
        return (char)(year%400);
    else
        return (char)(year%4);
}         

//  Real Time Clock interrupts every 4msec - This routine decrements 
//  countdown timers and maintains the watch clock
ISR(TIMER2_COMPA_vect)
{
 	if(rtccountdown1>0)  rtccountdown1--; 	
//	if(rtccountdown3>0)  rtccountdown3--;
	
	if(++rtcclockcnt==0) {
		if(rtccountdown4>0)  rtccountdown4--;
		if(rtccountdown2>0)  rtccountdown2--;
	    if(seconds++>=59) {
		    seconds = 0;
			if(minutes++>=59) {
			    minutes = 0;
				if(hours++>=23) {
				    hours = 0;
					days++;
                    /*if(days==31){
                        month++;
                        days = 1;
                		}
                	else if(days==30) {                    
                        if((month==4) || (month==6) || (month==9) || (month==11)) {
                            month++;
                            days = 1;
                            }
                        }
                    else if(days==29) {
                        if(month==2) {
                            month++;
                            days = 1;
                            }
                        }              
                    else if(days==28) {
                        if((month==2) && (not_leap())) {
                            month++;
                            days = 1;
                    		}                
                		}                          
                	if(month==13) {
                        month = 1;
                        year++;
                        }
					*/
					}
				}
			}
		}
}
