/**************************************************************

   Program Title:  IMU64X0 Data Logger
   
   Program Name:  IMU64X0_Uart.c	  Version: 1.00
   
   Author:  SS	  					  Date:  Feb 22, 2013
   
   Compiled with AVR Studio 4.19.
   
   This file contains routines that implement basic Usart put/get functions.
   
   Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2004-2013, SOC Robotics, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 * 4. Redistribution is only allowed in combination with SOC Robotics, 
 *    Inc. hardware products.
 *
 * THIS SOFTWARE IS PROVIDED BY SOC ROBOTICS, INC. AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.soc-robotics.com
 */

/*
 * $Log: IMU64X0_Uart.c,v $
 * Revision 1.00  2013/02/22 SS
 *    Support for ZigBee added
 * Revision 0.86  2006/02/07 16:40:20  SS
 *    2-wire interface, 8MHz clock, AVR Flash writer, XON/XOFF, 38,700baud
 * Revision 0.52  2005/06/04 18:36:42  SS
 * Preview release
 *
 */
// Include header files for GCC/IAR
#include "avr_compiler.h"
#include <avr/pgmspace.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "IMU64X0_Uart.h"
#include "IMU64X0_Defines.h"
#include "IMU64X0_Type.h"
#include "IMU64X0_Extern.h"


int PutChar(unsigned char indata)
{
	unsigned char tmphead;
	
	// Calculate buffer index
	tmphead = (UART_TxHead + 1) & UART_TX_BUFFER_MASK; 
		
	// Wait for free space in buffer
	while(tmphead == UART_TxTail) ;
	
	UART_TxBuf[tmphead] = indata; // store data in buffer
	UART_TxHead = tmphead;      // store new index

	// Expand LF to CRLF is textmode globel set
	if((indata==LF)&&(_textmode==1)) {
		// Calculate buffer index
		tmphead = (UART_TxHead + 1) & UART_TX_BUFFER_MASK; 
		
		// Wait for free space in buffer
		while(tmphead == UART_TxTail) ;
	
		UART_TxBuf[tmphead] = CR; // store data in buffer
		UART_TxHead = tmphead;      // store new index
		}

	UCSR0B |= (1<<UDRIE0);        // Fire TX UDRIE interrupt
	
	return 1;
}

int putchar1(unsigned char indata)
{
	unsigned char tmphead;
	
	// Calculate buffer index
	tmphead = (UART_TxHead1 + 1) & UART_TX_BUFFER_MASK; 
		
	// Wait for free space in buffer
	while(tmphead == UART_TxTail1) ;
	
	UART_TxBuf1[tmphead] = indata; // store data in buffer
	UART_TxHead1 = tmphead;      // store new index

	// Expand LF to CRLF if textmode global set
	if((indata==LF)&&(_textmode==1)) {
		// Calculate buffer index
		tmphead = (UART_TxHead1 + 1) & UART_TX_BUFFER_MASK; 
		
		// Wait for free space in buffer
		while(tmphead == UART_TxTail1) ;
	
		UART_TxBuf1[tmphead] = CR; // store data in buffer
		UART_TxHead1 = tmphead;      // store new index
		}

	UCSR1B |= (1<<UDRIE1);        // Fire TX UDRIE interrupt
	
	return 1;
}

void putcharr(unsigned char indata)
{
 	putchar(indata);
}

void putcharr1(unsigned char indata)
{
 	putchar1(indata);
}

// Read and write functions
unsigned char getcharnw(void)
{
	unsigned char tmptail;

	if(UART_RxHead==UART_RxTail) return 0; // nothing to get
		
	tmptail = (UART_RxTail+1)&UART_RX_BUFFER_MASK;  // calculate buffer index
	UART_RxTail = tmptail;     // store new index
	
	rxcnt--;
	if(rxcnt<0) rxcnt = 0;


	return UART_RxBuf[tmptail];   // return data
}

// Read and write functions
unsigned char getcharnw1(void)
{
	unsigned char tmptail;

	if(UART_RxHead1==UART_RxTail1) return 0; // nothing to get
		
	tmptail = (UART_RxTail1+1)&UART_RX_BUFFER_MASK;  // calculate buffer index
	UART_RxTail1 = tmptail;     // store new index
	
	rxcnt1--;
	if(rxcnt1<0) rxcnt1 = 0;

	return UART_RxBuf1[tmptail];   // return data
}

// Getchar waits until a valid character is received
unsigned char GetChar(void)
{
    unsigned char inchar;
	
 	while(!(inchar=getcharnw())) ;
	if(echoinchars==YES) putcharr(inchar);
	return inchar;
}

// Getchar waits until a valid character is received
unsigned char getchar1(void)
{
    unsigned char inchar;
	
 	while(!(inchar=getcharnw1())) ;
	if(echoinchars==YES) putcharr1(inchar);
	return inchar;
}

// Getchar waits until a valid character is received
unsigned char getcharr(void)
{
    unsigned char inchar;
	
 	while(!(inchar=getcharnw())) ;
	if(echoinchars==YES) putcharr(inchar);
	return inchar;
}

/* Initialize UART */
void InittUART(unsigned char baudrate)
{
 	/* set the baud rate */
	UBRR0H = 0x00;
	UBRR0L = baudrate; 
	UCSR0B = BIT(RXEN0) | BIT(TXEN0); /* enable UART receiver and transmitter */
    //  8 data bits, 2 stop bits
//	UCSRC = (1<<URSEL)|(1<<USBS)|(3<<UCSZ0);
    //  8 data bits, 1 stop bit
	UCSR0C = (1<<UMSEL00)|(3<<UCSZ00);
}

// Polled receive function - non-wait
unsigned char ReceiveByte(void)
{
	if((UCSR0A & (1<<RXC0))) return UDR0;
	else return 0x00;
}

// Polled transmit function
void TransmitByte(unsigned char data)
{
	while ( !(UCSR0A & (1<<UDRE0)) )
		; /* wait for empty transmit buffer */
	UDR0 = data; /* start transmission */
}

//  Buffered write to UART - wait until transmitted
void puttchar(unsigned char data)
{
 	 TransmitByte(data);
}

//  Buffered character read from UART - don't return if 
//  character is Null
unsigned char gettchar(void)
{
     unsigned char inchar;
	 
 	 while(!(inchar=ReceiveByte())) ;
 	 return inchar;
}


void putss(char *outstrg)
{
 	 int i;
	 
	 for(i=0; i<strlen(outstrg); i++)  putcharr(outstrg[i]);
}

void puts1(char *outstrg)
{
 	 int i;
	 
	 for(i=0; i<strlen(outstrg); i++)  putchar1(outstrg[i]);
}

const char putcrmess[] PROGMEM = "\n\r";
const char putcrpmess[] PROGMEM = "\n\r-";

void putCRLF(void)
{
 	 putfs(putcrmess);
}

void putCRLFP(void)
{
 	 putfs(putcrpmess);
}


//  Load string constant from Flash (conserves RAM)
void putfs(const char *signon1)
{ 	
	char inchar;
    int i;
	
	i = 0;
	while((inchar=pgm_read_byte(&signon1[i++]))!=0x00)  putcharr(inchar);
}

//  Convert byte to ASCII hex
void bytetohex(unsigned char byte)
{
    unsigned char inbyte;
	
	inbyte = byte & 0x0f;
	if(inbyte > 9) hexbyte[1] = (inbyte - 9) + 0x40;
	else hexbyte[1] = 0x30 + inbyte;
	inbyte = byte & 0xf0;
	inbyte >>= 4;
	if(inbyte >9) hexbyte[0] = (inbyte - 9) + 0x40;
	else hexbyte[0] = 0x30 + inbyte;
    hexbyte[2] = 0x00;
}

// Convert word to ASCII Hex
void addresstohex(unsigned int address)
{
 	 unsigned char lowbyte,highbyte;
	 
	 lowbyte = address & 0x00FF;
	 bytetohex(lowbyte);
	 hexaddress[2] = hexbyte[0];
	 hexaddress[3] = hexbyte[1];
	 highbyte = address/256;
	 bytetohex(highbyte);
	 hexaddress[0] = hexbyte[0];
	 hexaddress[1] = hexbyte[1];
	 hexaddress[4] = 0x00;
}

// Convert ASCII hex to binary
unsigned char converttohex(char *instrg)
{
 	 unsigned char lownibble, highnibble;
	 
	 if((instrg[0] >= 'A')&(instrg[0]<='F')) highnibble = instrg[0] - 'A' + 10;
	 else if((instrg[0] >= 'a')&(instrg[0]<='f')) highnibble = instrg[0] - 'a' + 10;
	 else highnibble = instrg[0] - '0';
	 
	 if((instrg[1] >= 'A')&(instrg[1]<='F')) lownibble = instrg[1] - 'A' + 10;
	 else if((instrg[1] >= 'a')&(instrg[1]<='f')) lownibble = instrg[1] - 'a' + 10;
	 else lownibble = instrg[1] - '0';
	 
	 highnibble = highnibble<<4;
	 return highnibble + lownibble;
}

